/*
**	2007 Leonid Tsinitskiy (LionTs)
**	Simple image viewer with slideshow ability
**		this sample covers:
**			- get information about a package contents
**			- display package contents to select an item
**			- load selected item only from a package
**		http://www.Lion-Ts.com
*/

/*
**	include code
*/
include <custres.wdl>;	//use custom resource functions

/*
**	bind resources
*/
bind <logolite.gsr>;	//include custom resources in published game
bind <resource.gsr>;

/*
**	defines, variables, strings, texts, etc.
*/
plugindir = ".";

var video_mode = 7; // 800x600
var video_depth = 32; // 32 bit mode
var video_screen = 2;  //windowed mode
var wndf=0;	//for Alt+Tab processing
var TotalFiles = 0;	//file count in a resource
var Choice = 0;	//selected image

function* old_on_enter;	//Alt+Enter handler

panel* wait_pan;		// logo panel placeholder

//bmap* imgTemp;	//temp image

string level_wmb = <ImgView.wmb>;
string res_demo  = "Custom Resources for 3DGS A6.4+\nDemo #7 by LionTs,\nLook at main.wdl and custres.wdl for details\n"; 
string help_str = "Up,Down-select    Enter-view   A-auto    F10-exit"; 
string WndCaption = "CR Pack Demo #7 by LionTs";
string strChoice;	//selected image name
string strTemp;	//temp string
string strTemp1;	//temp string
font skinc_font = "font.tga", 16, 23;

text res_demo_txt	// displays "Demo..."
{
	pos_x = 25;
	pos_y = 0;
	layer = 100;
	font = skinc_font;
	string = res_demo;
}
text help_txt		// displays help string
{
	pos_x = 0;
	pos_y = 575;
	layer = 100;
	font = skinc_font;
	string = help_str;
}

text filelist_txt	// displays filelist
{
	pos_x = 200;
	pos_y = 200;
	layer = 100;
	font = skinc_font;
	strings = 10;
}

/*
**	Utility functions
*/
//our Alt+Enter handler
function new_on_enter
{
	old_on_enter();	//call old handler
	if (video_screen==2){	//setup window params
		video_window(nullvector, nullvector, 113, WndCaption);	//no resize & new wnd caption
	}
}
//exit function
function game_fin()
{
	proc_kill(4);					//for a case of logical errors in code
	CR_Close("resource.gsr", Off);	//you don't have to call it at the app exit, this is just an example
	sys_exit("");
}
//remove extensions from file list
function prepare_filelist
{
	var i=0;
	while (i < TotalFiles){
		str_trunc(filelist_txt.string[i], 4);
		str_cpy(strTemp, " ");
		str_cat(strTemp, filelist_txt.string[i]);
		str_cpy(filelist_txt.string[i], strTemp);
		i += 1;
	}
}
//prepare selected item
function prepare_item
{
	str_cpy(strTemp, filelist_txt.string[Choice]);	//copy to the temp storage
	str_clip(strTemp, 1);	//remove first character
	str_cat(strTemp, ".dds");	//add file extension (we have dds only in the package)
}
//add or remove marker for selected item
function add_marker(item, add)
{
	str_clip(filelist_txt.string[item], 1);	//remove first character
	if (add==On){	//prepare prefix string
		str_cpy(strTemp, "^");
	}else{
		str_cpy(strTemp, " ");
	}
	str_cat(strTemp, filelist_txt.string[item]);	//add prefix (pointer character or space) to the string
	str_cpy(filelist_txt.string[item], strTemp);	//copy back
}
//center the image
function center_image
{
	wait_pan.pos_x = (screen_size.x - bmap_width(wait_pan.bmap))/2;	//center hor
	wait_pan.pos_y = (screen_size.y - bmap_height(wait_pan.bmap))/2;	//center vert
}
//create image and center it
function create_and_center_image
{
	str_cpy(strTemp1, "bmap=");	//prepare string for pan_create
	str_cat(strTemp1, strTemp);
	str_cat(strTemp1, ";pos_x=100;pos_y=100; alpha=100;");
	wait_pan = pan_create(strTemp1, 50);	//create the panel with loaded image
	center_image();
	wait_pan.visible = on;	//show the image
}
//remove image
function remove_image
{
	wait_pan.visible = off;	//hide the image
	pan_remove(wait_pan);	//remove panel with image
}
/*
**	Starters
*/
//pause game on loose focus
function pause_game_at_alt_tab_startup()
{
	while (1)
	{
		if (window_focus == 0){
			if (!wndf){
				wndf=1;
				camera.visible=off;	//for no cpu usage
				fps_max=4;
				freeze_mode = 1;
			}
		}else{
			if (wndf){
				freeze_mode = 0;
				fps_max=16;
				camera.visible=on;
				wndf=0;
			}
		}
		wait(1);
	}
}
//change Alt+Enter handler at start
function save_on_enter_handler_startup
{
	old_on_enter=on_enter;	//save old handler
	on_enter=new_on_enter;	//assign new handler
}

/*
**	main entrypoint
*/
function main()
{
	var i;	//for a case ;)
	var j;
	//black background
	sky_color.red = 10;
	sky_color.green = 10;
	sky_color.blue = 10;
	//change window caption
	if (video_screen==2){
		
		video_window(nullvector, nullvector, 113, WndCaption);	//no resize & new wnd caption
	}
	camera.visible=off;
	fps_max=4;
	freeze_mode = 1;

	CR_Open("logolite.gsr", "", On);	//open custom resource
	CR_Open("resource.gsr", "", On);	//open custom resource
	CR_LoadAll("logolite.gsr", On);	//load all splash screens
	wait_pan = pan_create("bmap = logolite.tga; alpha=50;", 1000);
	center_image();
	wait_pan.visible = on;
	wait(3);
	clip_size = 0; // show all the triangles for all the models
	fps_max = 40; // lock the frame rate to 30 fps
	fps_min=16;
	level_load(level_wmb);	//load compiled level
	wait (3); // wait for the level to be loaded
	
	wait(-1);	//delay to display Conitec's logo for a 1 sec at least
	
	//display level in view
	camera.visible=on;
	res_demo_txt.visible = on;
	freeze_mode = 0;
	wait(3);
	freeze_mode = 1;
	//dissolving logo
	wait_pan.transparent = on;
	while (wait_pan.alpha > 0)
	{
		wait_pan.alpha = max(wait_pan.alpha - 7 * time_step, 0);
		wait(1);
	}
	wait_pan.visible = off;
	
	pan_remove(wait_pan);	// remove splashscreen

	wait_pan = pan_create("bmap = liontslogo.tga; alpha=100;", 1000);
	center_image();
	wait_pan.visible = on;
	wait(-3);

	var j;
	j = wait_pan.scale_y / wait_pan.scale_x;
	//disappearing logo
	while(wait_pan.scale_x > 0.1){
		wait_pan.scale_x = max(0.1, wait_pan.scale_x - 0.1 * time_step);
		wait_pan.scale_y = wait_pan.scale_x * j;
		wait_pan.pos_x = (screen_size.x - bmap_width(wait_pan.bmap) * wait_pan.scale_x)/2;
		wait_pan.pos_y = (screen_size.y - bmap_height(wait_pan.bmap) * wait_pan.scale_y)/2;
		wait(1);
	}
	wait_pan.visible = off;
	
	pan_remove(wait_pan);	// remove splashscreen
	CR_Close("logolite.gsr", On);	//unload all splash screens
	fps_max=16;
	freeze_mode = 0;
	
	TotalFiles = CR_FileCount("resource.gsr", On);	//get number of files
	//just exit if too many files, you may write more better solution
	if (TotalFiles > filelist_txt.strings){
		error("Too many files in the resource");
		game_fin();
	}
	CR_FileList("resource.gsr", filelist_txt, On);	//get list of files in the package
	
	prepare_filelist();	//want display names only
	add_marker(Choice, On);	//show cursor
	
	help_txt.visible = on;	//show help string
	filelist_txt.visible = on;	//show file list

//current version of A6 don't want to free all memory after sequence:
// pan_create->pan_remove and we have predictable memory leak
// however, may be I did something wrong here...
// generally, try it by yourself.
/*
while(1){	//test cycle to check for memory leaks without pan_create/pan_remove
CR_LoadFile("resource.gsr", "The_Hunter.dds", On);	//load image file
wait(1);
CR_UnLoadFile("resource.gsr", "The_Hunter.dds", On);	//unload image file (free memory)
wait(1);
}
*/

	//main application loop
	while(1){
		while (key_any==1){	//wait for key release
			wait(1);
		}
		//wait for a keypress
		while (!key_a && !key_cul && !key_cur && !key_cuu && !key_cud && !key_enter && !key_space){
			wait(1);
		}

		if (key_cuu || key_cul){	//is the cursor up?
			while (key_any==1){	//wait for key release
				wait(1);
			}
			add_marker(Choice, Off);	//hide cursor
			Choice -= 1;	//"goes up"
			Choice = max(Choice, 0);	//check for the limit
			add_marker(Choice, On);	//show cursor
		}else{
		if (key_cud || key_cur){	//is the cursor down?
			while (key_any==1){	//wait for a key release
				wait(1);
			}
			add_marker(Choice, Off);	//hide cursor
			Choice += 1;	//"goes down"
			Choice = min(Choice, TotalFiles - 1);	//check for the limit
			add_marker(Choice, On);	//show cursor
		}else{
		if (key_enter || key_space){	//is the enter or space?
			while (key_any==1){	//wait for a key release
				wait(1);
			}
			filelist_txt.visible = off;	//hide filelist
			prepare_item();	//prepare item
			CR_LoadFile("resource.gsr", strTemp, On);	//load image file
			create_and_center_image();
			while (key_any==0){	//wait for a key press
				wait(1);
			}
			remove_image();
			CR_UnLoadFile("resource.gsr", strTemp, On);	//unload image file (free memory)
			filelist_txt.visible = on;	//show filelist
			while (key_any==1){	//wait for a key release
				wait(1);
			}
		}else{
		if (key_a){	//is the A key?
			i = Choice;	//save current choice
			help_txt.visible = off;
			res_demo_txt.visible = off;
			while(1){	//do it forever
				while (key_any==1){	//wait for a key release
					wait(1);
				}
				filelist_txt.visible = off;	//hide filelist
				prepare_item();	//prepare item
				CR_LoadFile("resource.gsr", strTemp, On);	//load image file
				create_and_center_image();
				j = 0;
				while (j < 48){	//delay to show an image
					if (key_any){	//go out if a key pressed
						break;
					}
					j += time_step;
					wait(1);
				}
				remove_image();
				CR_UnLoadFile("resource.gsr", strTemp, On);	//unload image file (free memory)
				Choice += 1;	//"goes down"
				Choice = cycle(Choice, 0, TotalFiles);	//check for the limit
				if (key_any){	//go out if a key pressed
					filelist_txt.visible = on;	//show filelist
					help_txt.visible = on;
					res_demo_txt.visible = on;
					Choice = i;	//restore current choice
					break;
				}
				wait(1);
			}
		}}}}}
	}
}

/*
**	Key bindings
*/
//one way to exit
on_close = game_fin;
on_f10 = game_fin;
on_esc = NULL;